@@ -5,6 +5,7 @@
 
 from __future__ import annotations
 
+from contextlib import contextmanager
 from datetime import datetime, timezone
 import logging
 
@@ -34,6 +35,23 @@ class InputError(ValueError):
     pass
 
 
+@contextmanager
+def trap_and_report_exceptions():
+    """Trap and report exceptions (InputError, MaintenanceError) in a unified way.
+
+    """
+    from swh.deposit.client import MaintenanceError
+
+    try:
+        yield
+    except InputError as e:
+        logger.error("Problem during parsing options: %s", e)
+        sys.exit(1)
+    except MaintenanceError as e:
+        logger.error(e)
+        sys.exit(1)
+
+
 def generate_slug() -> str:
     """Generate a slug (sample purposes).
 
@@ -60,7 +78,7 @@ def _url(url: str) -> str:
 
 
 def generate_metadata(
-    deposit_client: str, name: str, external_id: str, authors: List[str]
+    deposit_client: str, name: str, external_id: Optional[str], authors: List[str]
 ) -> str:
     """Generate sword compliant xml metadata with the minimum required metadata.
 
@@ -94,11 +112,10 @@ def generate_metadata(
     import xmltodict
 
     # generate a metadata file with the minimum required metadata
-    codemetadata = {
-        "entry": {
+    document = {
+        "atom:entry": {
             "@xmlns:atom": "http://www.w3.org/2005/Atom",
             "@xmlns:codemeta": "https://doi.org/10.5063/SCHEMA/CODEMETA-2.0",
-            "codemeta:identifier": external_id,
             "atom:updated": datetime.now(tz=timezone.utc),  # mandatory, cf. docstring
             "atom:author": deposit_client,  # mandatory, cf. docstring
             "atom:title": name,  # mandatory, cf. docstring
@@ -108,8 +125,11 @@ def generate_metadata(
             ],
         },
     }
-    logging.debug("Metadata dict to generate as xml: %s", codemetadata)
-    return xmltodict.unparse(codemetadata, pretty=True)
+    if external_id:
+        document["atom:entry"]["codemeta:identifier"] = external_id
+
+    logging.debug("Atom entry dict to generate as xml: %s", document)
+    return xmltodict.unparse(document, pretty=True)
 
 
 def _collection(client: PublicApiDepositClient) -> str:
@@ -120,7 +140,9 @@ def _collection(client: PublicApiDepositClient) -> str:
     sd_content = client.service_document()
     if "error" in sd_content:
         raise InputError("Service document retrieval: %s" % (sd_content["error"],))
-    collection = sd_content["service"]["workspace"]["collection"]["sword:name"]
+    collection = sd_content["app:service"]["app:workspace"]["app:collection"][
+        "sword:name"
+    ]
     return collection
 
 
@@ -181,10 +203,6 @@ def client_command_parse_input(
             "swhid": optional deposit swhid
             "replace": whether the given deposit is to be replaced or not
     """
-
-    if not slug:  # generate one as this is mandatory
-        slug = generate_slug()
-
     if not metadata:
         if name and authors:
             metadata_path = os.path.join(temp_dir, "metadata.xml")
@@ -251,11 +269,41 @@ def _subdict(d: Dict[str, Any], keys: Collection[str]) -> Dict[str, Any]:
     return {k: v for k, v in d.items() if k in keys}
 
 
+def credentials_decorator(f):
+    """Add default --url, --username and --password flag to cli.
+
+    """
+    f = click.option(
+        "--password", required=True, help="(Mandatory) User's associated password"
+    )(f)
+    f = click.option("--username", required=True, help="(Mandatory) User's name")(f)
+    f = click.option(
+        "--url",
+        default="https://deposit.softwareheritage.org",
+        help=(
+            "(Optional) Deposit server api endpoint. By default, "
+            "https://deposit.softwareheritage.org/1"
+        ),
+    )(f)
+    return f
+
+
+def output_format_decorator(f):
+    """Add --format output flag decorator to cli.
+
+    """
+    return click.option(
+        "-f",
+        "--format",
+        "output_format",
+        default="logging",
+        type=click.Choice(["logging", "yaml", "json"]),
+        help="Output format results.",
+    )(f)
+
+
 @deposit.command()
-@click.option("--username", required=True, help="(Mandatory) User's name")
-@click.option(
-    "--password", required=True, help="(Mandatory) User's associated password"
-)
+@credentials_decorator
 @click.option(
     "--archive",
     type=click.Path(exists=True),
@@ -268,7 +316,7 @@ def _subdict(d: Dict[str, Any], keys: Collection[str]) -> Dict[str, Any]:
         "(Optional) Path to xml metadata file. If not provided, "
         "this will use a file named <archive>.metadata.xml"
     ),
-)  # noqa
+)
 @click.option(
     "--archive-deposit/--no-archive-deposit",
     default=False,
@@ -282,14 +330,14 @@ def _subdict(d: Dict[str, Any], keys: Collection[str]) -> Dict[str, Any]:
 @click.option(
     "--collection",
     help="(Optional) User's collection. If not provided, this will be fetched.",
-)  # noqa
+)
 @click.option(
     "--slug",
     help=(
         "(Optional) External system information identifier. "
         "If not provided, it will be generated"
     ),
-)  # noqa
+)
 @click.option(
     "--partial/--no-partial",
     default=False,
@@ -297,12 +345,12 @@ def _subdict(d: Dict[str, Any], keys: Collection[str]) -> Dict[str, Any]:
         "(Optional) The deposit will be partial, other deposits "
         "will have to take place to finalize it."
     ),
-)  # noqa
+)
 @click.option(
     "--deposit-id",
     default=None,
     help="(Optional) Update an existing partial deposit with its identifier",
-)  # noqa
+)
 @click.option(
     "--swhid",
     default=None,
@@ -312,15 +360,7 @@ def _subdict(d: Dict[str, Any], keys: Collection[str]) -> Dict[str, Any]:
     "--replace/--no-replace",
     default=False,
     help="(Optional) Update by replacing existing metadata to a deposit",
-)  # noqa
-@click.option(
-    "--url",
-    default="https://deposit.softwareheritage.org",
-    help=(
-        "(Optional) Deposit server api endpoint. By default, "
-        "https://deposit.softwareheritage.org/1"
-    ),
-)  # noqa
+)
 @click.option("--verbose/--no-verbose", default=False, help="Verbose mode")
 @click.option("--name", help="Software name")
 @click.option(
@@ -329,14 +369,7 @@ def _subdict(d: Dict[str, Any], keys: Collection[str]) -> Dict[str, Any]:
     help="Software author(s), this can be repeated as many times"
     " as there are authors",
 )
-@click.option(
-    "-f",
-    "--format",
-    "output_format",
-    default="logging",
-    type=click.Choice(["logging", "yaml", "json"]),
-    help="Output format results.",
-)
+@output_format_decorator
 @click.pass_context
 def upload(
     ctx,
@@ -368,7 +401,7 @@ https://docs.softwareheritage.org/devel/swh-deposit/getting-started.html.
     """
     import tempfile
 
-    from swh.deposit.client import MaintenanceError, PublicApiDepositClient
+    from swh.deposit.client import PublicApiDepositClient
 
     if archive_deposit or metadata_deposit:
         warnings.warn(
@@ -383,7 +416,7 @@ https://docs.softwareheritage.org/devel/swh-deposit/getting-started.html.
 
     client = PublicApiDepositClient(url=url, auth=(username, password))
     with tempfile.TemporaryDirectory() as temp_dir:
-        try:
+        with trap_and_report_exceptions():
             logger.debug("Parsing cli options")
             config = client_command_parse_input(
                 client,
@@ -401,12 +434,6 @@ https://docs.softwareheritage.org/devel/swh-deposit/getting-started.html.
                 author,
                 temp_dir,
             )
-        except InputError as e:
-            logger.error("Problem during parsing options: %s", e)
-            sys.exit(1)
-        except MaintenanceError as e:
-            logger.error(e)
-            sys.exit(1)
 
         if verbose:
             logger.info("Parsed configuration: %s", config)
@@ -422,43 +449,21 @@ https://docs.softwareheritage.org/devel/swh-deposit/getting-started.html.
 
 
 @deposit.command()
-@click.option(
-    "--url",
-    default="https://deposit.softwareheritage.org",
-    help="(Optional) Deposit server api endpoint. By default, "
-    "https://deposit.softwareheritage.org/1",
-)
-@click.option("--username", required=True, help="(Mandatory) User's name")
-@click.option(
-    "--password", required=True, help="(Mandatory) User's associated password"
-)
+@credentials_decorator
 @click.option("--deposit-id", default=None, required=True, help="Deposit identifier.")
-@click.option(
-    "-f",
-    "--format",
-    "output_format",
-    default="logging",
-    type=click.Choice(["logging", "yaml", "json"]),
-    help="Output format results.",
-)
+@output_format_decorator
 @click.pass_context
 def status(ctx, url, username, password, deposit_id, output_format):
     """Deposit's status
 
     """
-    from swh.deposit.client import MaintenanceError, PublicApiDepositClient
+    from swh.deposit.client import PublicApiDepositClient
 
     url = _url(url)
     logger.debug("Status deposit")
-    try:
+    with trap_and_report_exceptions():
         client = PublicApiDepositClient(url=url, auth=(username, password))
         collection = _collection(client)
-    except InputError as e:
-        logger.error("Problem during parsing options: %s", e)
-        sys.exit(1)
-    except MaintenanceError as e:
-        logger.error(e)
-        sys.exit(1)
 
     print_result(
         client.deposit_status(collection=collection, deposit_id=deposit_id),
@@ -480,3 +485,37 @@ def print_result(data: Dict[str, Any], output_format: Optional[str]) -> None:
         click.echo(yaml.dump(data))
     else:
         logger.info(data)
+
+
+@deposit.command("metadata-only")
+@credentials_decorator
+@click.option(
+    "--metadata",
+    "metadata_path",
+    type=click.Path(exists=True),
+    required=True,
+    help="Path to xml metadata file",
+)
+@output_format_decorator
+@click.pass_context
+def metadata_only(ctx, url, username, password, metadata_path, output_format):
+    """Deposit metadata only upload
+
+    """
+    from swh.deposit.client import PublicApiDepositClient
+    from swh.deposit.utils import parse_swh_reference, parse_xml
+
+    # Parse to check for a swhid presence within the metadata file
+    with open(metadata_path, "r") as f:
+        metadata_raw = f.read()
+    actual_swhid = parse_swh_reference(parse_xml(metadata_raw))
+
+    if not actual_swhid:
+        raise InputError("A SWHID must be provided for a metadata-only deposit")
+
+    with trap_and_report_exceptions():
+        client = PublicApiDepositClient(url=_url(url), auth=(username, password))
+        collection = _collection(client)
+        result = client.deposit_metadata_only(collection, metadata_path)
+
+    print_result(result, output_format)
