@@ -5,6 +5,7 @@
 
 import base64
 from functools import partial
+from io import BytesIO
 import os
 import re
 from typing import Mapping
@@ -28,7 +29,7 @@ from swh.deposit.config import (
     DEPOSIT_STATUS_PARTIAL,
     DEPOSIT_STATUS_REJECTED,
     DEPOSIT_STATUS_VERIFIED,
-    EDIT_SE_IRI,
+    SE_IRI,
     setup_django_for,
 )
 from swh.deposit.parsers import parse_xml
@@ -50,6 +51,16 @@ TEST_USER = {
 }
 
 
+ANOTHER_TEST_USER = {
+    "username": "test2",
+    "password": "password2",
+    "email": "test@example2.org",
+    "provider_url": "https://hal-test.archives-ouvertes.example/",
+    "domain": "archives-ouvertes.example/",
+    "collection": {"name": "another-collection"},
+}
+
+
 def pytest_configure():
     setup_django_for("testing")
 
@@ -174,28 +185,37 @@ deposit_collection = deposit_collection_factory()
 deposit_another_collection = deposit_collection_factory("another-collection")
 
 
-@pytest.fixture
-def deposit_user(db, deposit_collection):
+def _create_deposit_user(db, collection, user_data):
     """Create/Return the test_user "test"
 
     """
     from swh.deposit.models import DepositClient
 
     try:
-        user = DepositClient._default_manager.get(username=TEST_USER["username"])
+        user = DepositClient._default_manager.get(username=user_data["username"])
     except DepositClient.DoesNotExist:
         user = DepositClient._default_manager.create_user(
-            username=TEST_USER["username"],
-            email=TEST_USER["email"],
-            password=TEST_USER["password"],
-            provider_url=TEST_USER["provider_url"],
-            domain=TEST_USER["domain"],
+            username=user_data["username"],
+            email=user_data["email"],
+            password=user_data["password"],
+            provider_url=user_data["provider_url"],
+            domain=user_data["domain"],
         )
-        user.collections = [deposit_collection.id]
+        user.collections = [collection.id]
         user.save()
     return user
 
 
+@pytest.fixture
+def deposit_user(db, deposit_collection):
+    return _create_deposit_user(db, deposit_collection, TEST_USER)
+
+
+@pytest.fixture
+def deposit_another_user(db, deposit_another_collection):
+    return _create_deposit_user(db, deposit_another_collection, ANOTHER_TEST_USER)
+
+
 @pytest.fixture
 def client():
     """Override pytest-django one which does not work for djangorestframework.
@@ -204,23 +224,35 @@ def client():
     return APIClient()  # <- drf's client
 
 
-@pytest.fixture
-def authenticated_client(client, deposit_user):
+def _create_authenticated_client(client, user, user_data):
     """Returned a logged client
 
     This also patched the client instance to keep a reference on the associated
     deposit_user.
 
     """
-    _token = "%s:%s" % (deposit_user.username, TEST_USER["password"])
+    _token = "%s:%s" % (user.username, user_data["password"])
     token = base64.b64encode(_token.encode("utf-8"))
     authorization = "Basic %s" % token.decode("utf-8")
     client.credentials(HTTP_AUTHORIZATION=authorization)
-    client.deposit_client = deposit_user
+    client.deposit_client = user
     yield client
     client.logout()
 
 
+@pytest.fixture
+def authenticated_client(client, deposit_user):
+    yield from _create_authenticated_client(client, deposit_user, TEST_USER)
+
+
+@pytest.fixture
+def another_authenticated_client(deposit_another_user):
+    client = APIClient()
+    yield from _create_authenticated_client(
+        client, deposit_another_user, ANOTHER_TEST_USER
+    )
+
+
 @pytest.fixture
 def sample_archive(tmp_path):
     """Returns a sample archive
@@ -262,6 +294,7 @@ def create_deposit(
     sample_archive,
     external_id: str,
     deposit_status=DEPOSIT_STATUS_DEPOSITED,
+    in_progress=False,
 ):
     """Create a skeleton shell deposit
 
@@ -277,15 +310,17 @@ def create_deposit(
         HTTP_SLUG=external_id,
         HTTP_CONTENT_MD5=sample_archive["md5sum"],
         HTTP_PACKAGING="http://purl.org/net/sword/package/SimpleZip",
-        HTTP_IN_PROGRESS="false",
+        HTTP_IN_PROGRESS=str(in_progress).lower(),
         HTTP_CONTENT_DISPOSITION="attachment; filename=%s" % (sample_archive["name"]),
     )
 
     # then
-    assert response.status_code == status.HTTP_201_CREATED
+    assert response.status_code == status.HTTP_201_CREATED, response.content.decode()
     from swh.deposit.models import Deposit
 
-    deposit = Deposit._default_manager.get(external_id=external_id)
+    response_content = parse_xml(BytesIO(response.content))
+    deposit_id = response_content["swh:deposit_id"]
+    deposit = Deposit._default_manager.get(id=deposit_id)
 
     if deposit.status != deposit_status:
         deposit.status = deposit_status
@@ -297,10 +332,9 @@ def create_deposit(
 def create_binary_deposit(
     authenticated_client,
     collection_name: str,
-    sample_archive,
-    external_id: str,
     deposit_status: str = DEPOSIT_STATUS_DEPOSITED,
     atom_dataset: Mapping[str, bytes] = {},
+    **kwargs,
 ):
     """Create a deposit with both metadata and archive set. Then alters its status
        to `deposit_status`.
@@ -309,16 +343,16 @@ def create_binary_deposit(
     deposit = create_deposit(
         authenticated_client,
         collection_name,
-        sample_archive,
-        external_id=external_id,
         deposit_status=DEPOSIT_STATUS_PARTIAL,
+        **kwargs,
     )
 
+    origin_url = deposit.client.provider_url + deposit.external_id
+
     response = authenticated_client.post(
-        reverse(EDIT_SE_IRI, args=[collection_name, deposit.id]),
+        reverse(SE_IRI, args=[collection_name, deposit.id]),
         content_type="application/atom+xml;type=entry",
-        data=atom_dataset["entry-data0"] % deposit.external_id.encode("utf-8"),
-        HTTP_SLUG=deposit.external_id,
+        data=atom_dataset["entry-data0"] % origin_url,
         HTTP_IN_PROGRESS="true",
     )
 
@@ -328,15 +362,12 @@ def create_binary_deposit(
     from swh.deposit.models import Deposit
 
     deposit = Deposit._default_manager.get(pk=deposit.id)
-    if deposit.status != deposit_status:
-        deposit.status = deposit_status
-        deposit.save()
 
     assert deposit.status == deposit_status
     return deposit
 
 
-def deposit_factory(deposit_status=DEPOSIT_STATUS_DEPOSITED):
+def deposit_factory(deposit_status=DEPOSIT_STATUS_DEPOSITED, in_progress=False):
     """Build deposit with a specific status
 
     """
@@ -355,6 +386,7 @@ def deposit_factory(deposit_status=DEPOSIT_STATUS_DEPOSITED):
             sample_archive,
             external_id=external_id,
             deposit_status=deposit_status,
+            in_progress=in_progress,
         )
 
     return _deposit
@@ -362,7 +394,9 @@ def deposit_factory(deposit_status=DEPOSIT_STATUS_DEPOSITED):
 
 deposited_deposit = deposit_factory()
 rejected_deposit = deposit_factory(deposit_status=DEPOSIT_STATUS_REJECTED)
-partial_deposit = deposit_factory(deposit_status=DEPOSIT_STATUS_PARTIAL)
+partial_deposit = deposit_factory(
+    deposit_status=DEPOSIT_STATUS_PARTIAL, in_progress=True
+)
 verified_deposit = deposit_factory(deposit_status=DEPOSIT_STATUS_VERIFIED)
 completed_deposit = deposit_factory(deposit_status=DEPOSIT_STATUS_LOAD_SUCCESS)
 failed_deposit = deposit_factory(deposit_status=DEPOSIT_STATUS_LOAD_FAILURE)
@@ -378,8 +412,9 @@ def partial_deposit_with_metadata(
     return create_binary_deposit(
         authenticated_client,
         deposit_collection.name,
-        sample_archive,
+        sample_archive=sample_archive,
         external_id="external-id-partial",
+        in_progress=True,
         deposit_status=DEPOSIT_STATUS_PARTIAL,
         atom_dataset=atom_dataset,
     )
@@ -401,7 +436,7 @@ def partial_deposit_only_metadata(
     assert response.status_code == status.HTTP_201_CREATED
 
     response_content = parse_xml(response.content)
-    deposit_id = response_content["deposit_id"]
+    deposit_id = response_content["swh:deposit_id"]
     from swh.deposit.models import Deposit
 
     deposit = Deposit._default_manager.get(pk=deposit_id)
